package com.cryptshare.examples.api;

import com.cryptshare.api.*;

import java.nio.file.Paths;
import java.util.Map;

public class ExampleMain {

	/**
	 * The Cryptshare Server URL
	 */
	private static final String SERVER_URL = "http://localhost";

	/**
	 * The sender's email address that is used for transfers.
	 */
	private static final String SENDER_EMAIL_ADDRESS = "cs-example@cryptshare.com";

	/**
	 * The absolute or relative path to the client store. The clients store is a file
	 * where client specific information (like verification data) is persisted with encryption.
	 */
	private static final String STORE_FILE_PATH = "client.store";

	private static final String[] EXAMPLE_FILE_PATHS = new String[] { "etc/sample-file_01.txt", "etc/sample-file_02.txt" };

	public static void main(String[] args) {
		try {
			System.out.println("Preparing Cryptshare client...");

			// Preparing the connection. The generics type parameter specifies the type of
			// file arguments we use later. In the current version, only java.lang.String
			// is supported.
			CryptshareConnection connection = new CryptshareConnection(new WebServiceUri(SERVER_URL));

			// Create a client object to perform transfers for a certain sender over a certain
			// Cryptshare server url. We also need to provide the path to our store file (see comments above).
			Client client = new Client(SENDER_EMAIL_ADDRESS, connection, Paths.get(STORE_FILE_PATH));

			// Ensure that we are using client mode. Please see
			// http://wiki.cryptshare.com/display/CSM/Add-On+Products
			// for the two different modes.
			// A unique ID is set and stored in the client store file. In client verification
			// mode, this id has to be added to the server's configuration.
			// Administration Interface > Add-On products > Cryptshare Java API
			CheckVerificationResult verificationStatus = client.checkVerification();
			if (verificationStatus.getVerificationMode().equals(VerificationMode.SENDER)) {
				System.err.println("Please configure your server to use client mode.");
				return;
			}

			if (!verificationStatus.isUserVerified()) {
				try {
					client.requestClientVerification();
					System.out.println("The client verification was successfully performed.");
				} catch (Exception e) {
					System.out.println(
							"The verification mode is set to 'Client', but your Client ID is not whitelisted. Please add your Client ID '"
									+ client.getClientId() + "' to the list of allowed clients.");
					return;
				}
			}

			// Now lets create a transfer. Again, we need to specify the type of file arguments.
			Transfer transfer = new Transfer();

			// Add the file by providing the path
			for (String exampleFilePath : EXAMPLE_FILE_PATHS) {
				transfer.addFile(new TransferFile(Paths.get(exampleFilePath)));
			}

			// Add a few recipients
			transfer.addRecipient(new Recipient("hans.mustermann@example.com"));
			transfer.addRecipient(new Recipient("erika.musterfrau@example.com"));

			// Complete the transfer's meta information...
			// ...for the Recipient
			transfer.setSubject("Example Subject");
			transfer.setMessage("This is an example message.");
			transfer.setRecipientLanguage(null);

			// ...for the Sender
			transfer.setSenderName("Example user");
			transfer.setSenderPhone("202-555-0161");
			transfer.setInformAboutDownload(false);

			// We ask the server to generate a password for the transfer according to the Cryptshare
			// server security settings.
			String password = client.requestGeneratedPassword(15);
			System.out.println("** USING PASSWORD: " + password);
			transfer.setPassword(password);

			// Default passwordMode is NONE.
			// As we are specifying the password for the transfer passwordMode needs to be set to MANUAL
			transfer.setPasswordMode(PasswordMode.MANUAL);

			// Finally, perform the transfer using the performTransfer method, which performs
			// the transfer synchronously.
			System.out.println("Starting Transfer...");
			client.performTransfer(transfer, new UploadProgressChangedHandler(), new UploadCompleteHandler() {
				@Override
				public void uploadComplete(Map<String, String> urlMappings, Map<String, String> smtpMappings, String serverGenPassword,
						TransferError transferError, String trackingId) {
					System.out.println("Transfer finished.");

					// Output individual file download links
					System.out.println("\nDownload links:");
					for (Map.Entry<String, String> recipientEntry : urlMappings.entrySet()) {
						// Map entry contains TransferID (key) and recipient address (value)
						System.out.println("Transfer ID: \"" + recipientEntry.getKey() + "\". Download Link: " + recipientEntry.getValue());
					}
				}
			}, new UploadInterruptedHandler(), new UploadCancelledHandler(), 1000);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
